@@ -1,4 +1,4 @@
-"spCopulaDDP" <- function(y, delta, x=NULL, s, prediction, prior, mcmc, state, FSA = TRUE, knots,
+"spCopulaDDP" <- function(y, delta, x=NULL, s, prediction, prior, mcmc, state, status=TRUE, FSA = TRUE, knots,
                     data=sys.frame(sys.parent()), na.action=na.fail, work.dir=NULL)
 UseMethod("spCopulaDDP")
 
@@ -11,6 +11,7 @@ function (y,
           prior, 
           mcmc,
           state,
+          status=TRUE,
           FSA = TRUE,
           knots,
           data=sys.frame(sys.parent()),
@@ -24,19 +25,21 @@ function (y,
   #########################################################################################
   # data structure and check if FSA is needed
   #########################################################################################
+  y <- as.vector(y); n <- length(y);
+  delta <- as.vector(delta);
+  X <- cbind(rep(1,n), x);
+  p <- ncol(X);
+  s <- t(s);
+  dnn <- .DistMat(s, s);
   if(FSA) {
     ss <- knots$ss; if(is.null(ss)) stop("please specify the knots vector ss if FSA is used");
     ss <- t(as.matrix(ss));
     id <- knots$blockid; if(is.null(id)) stop("please specify the bolock id that each observation is located in if FSA is used ");
-    orderindex = order(id); id = id[orderindex];
-    y <- as.vector(y); y = y[orderindex]; n = length(y);
-    delta <- as.vector(delta); delta = delta[orderindex];
-    X <- t(cbind(rep(1,n), x)); X = X[,orderindex]; p <- nrow(X);
-    s <- t(as.matrix(s)); s = s[,orderindex];
-    dnn <- .DistMat(s, s);
+    orderindex = order(id);
+    if(!(sum(orderindex==(1:n))==n)) stop("please sort the data by ID");
+    blocki = c( 0, cumsum(as.vector(table(id))) );
     dnm <- .DistMat(s, ss);
     dmm <- .DistMat(ss, ss);
-    blocki = c( 0, cumsum(as.vector(table(id))) );
     #########################################################################################
     # prediction
     #########################################################################################
@@ -58,12 +61,6 @@ function (y,
         ds0block[i,j] = (id[i]==predid[j])+0
       }
     }
-  } else{
-    y <- as.vector(y); n <- length(y);
-    delta <- as.vector(delta);
-    X <- t(cbind(rep(1,n), x)); p <- nrow(X);
-    s <- t(s);
-    dnn <- .DistMat(s, s);
   }
 
   #########################################################################################
@@ -93,7 +90,6 @@ function (y,
   #########################################################################################
   # initial analysis and mcmc parameters
   #########################################################################################
-  fit0 <- survival::survreg(formula = Surv(exp(y), delta) ~ t(X)-1, dist = "lognormal");
   nburn <- mcmc$nburn;
   nsave <- mcmc$nsave;
   nskip <- mcmc$nskip;
@@ -102,36 +98,59 @@ function (y,
   #########################################################################################
   # priors
   #########################################################################################
+  fit0 <- survival::survreg(formula = Surv(exp(y), delta) ~ x, dist = "lognormal");
+  #fit0=lm(y~x); sfit0=summary(fit0); sig2hat = sfit0$sigma^2; 
+  muhat = as.vector(fit0$coefficients);
+  sig2hat = fit0$scale^2
+  Sighat = as.matrix(fit0$var[(1:p),(1:p)]); Sigscale=100;
   N <- prior$N; if(is.null(N)) N <- 10;
-  m0 <- prior$m0; if(is.null(m0)) m0 <- rep(0,p);
-  S0 <- prior$S0; if(is.null(S0)) S0 <- diag(rep(1e5,p), nrow=p, ncol=p);
-  Sig0 <- prior$Sig0; if(is.null(Sig0)) Sig0 <- diag(rep(1e5,p), nrow=p, ncol=p);
-  k0 <- prior$k0; if(is.null(k0)) k0 <- 7;
+  m0 <- prior$m0; if(is.null(m0)) m0 <- muhat;
+  S0 <- prior$S0; if(is.null(S0)) S0 <- Sighat;
+  Sig0 <- prior$Sig0; if(is.null(Sig0)) Sig0 <- Sigscale*Sighat; #Sig0 <- diag(rep(1e4,p), nrow=p, ncol=p);
+  k0 <- prior$k0; if(is.null(k0)) k0 <- p+5;
   nua <-prior$nua; nub <- prior$nub;
-  if(is.null(nua)) nua=2; if(is.null(nub)) nub=1;
+  if(is.null(nua)) nua=2+1; #nua=2+sig2hat/4; 
+  if(is.null(nub)) nub=2*sig2hat; #nub=sig2hat/4*(nua-1);
   a0 <-prior$a0; b0 <- prior$b0;
-  if(is.null(a0)) a0=1; if(is.null(b0)) b0=1;
-  theta0 <- prior$theta0; if(is.null(theta0)) theta0 <- c(1.0, 1.0, 0.5, 0.2);
+  if(is.null(a0)) a0=2; if(is.null(b0)) b0=2;
+  theta0 <- prior$theta0; if(is.null(theta0)) theta0 <- c(5.0, 1.0, 1.0, 5.0);
   spl0 <- prior$spl0; if(is.null(spl0)) spl0 <- round(nburn/2);
-  spS0 <- prior$spS0; if(is.null(spS0)) spS0 <- diag(c(4,4));
-  spadapter <- prior$spadapter; if(is.null(spadapter)) spadapter <- (2.4)^2/2;
+  spS0 <- prior$spS0; if(is.null(spS0)) spS0 <- diag(c(0.25,0.1));
+  spadapter <- prior$spadapter; if(is.null(spadapter)) spadapter <- (2.38)^2/2;
 
   #########################################################################################
   # current state and mcmc specification
   #########################################################################################
-  mu <- state$mu; if(is.null(mu)) mu <- rep(0,p);
-  Sig <- state$Sig; if(is.null(Sig)) Sig <- diag(rep(1e5,p), nrow=p, ncol=p); 
-  beta<- state$beta; if(is.null(beta)) beta = matrix(coefficients(fit0), p, N);
-  sigma2<- state$sigma2; if(is.null(sigma2)) sigma2 = rep(fit0$scale,N);
-  alpha <- state$alpha; if(is.null(alpha)) alpha <- 1;
-  K = sample(1:N,n, replace=T);
-  V = rbeta(N, 1, alpha); V[N] =1;
-  w = V; 
-  for (k in 2:N){
-    w[k] = max( (1 - sum(w[1:(k-1)]))*V[k], 1e-20);
+  if(status){
+    currenty = y;
+    mu <- state$mu; if(is.null(mu)) mu <- muhat;
+    Sig <- state$Sig; if(is.null(Sig)) Sig <- 25*Sighat;
+    beta<- state$beta; if(is.null(beta)) beta = matrix(muhat, p, N);
+    sigma2<- state$sigma2; if(is.null(sigma2)) sigma2 = rep(sig2hat/2,N);
+    alpha <- state$alpha; if(is.null(alpha)) alpha <- 2;
+    K = sample(1:N,n, replace=T);
+    V = rbeta(N, 1, alpha); V[N] =1;
+    w = V; 
+    for (k in 2:N){
+      w[k] = max(exp( sum(log(1-V[1:(k-1)]))+log(V[k]) ), 1e-320);
+      #w[k] = max( (1 - sum(w[1:(k-1)]))*V[k], 1e-320);
+    }
+    theta = state$theta; if(is.null(theta)) theta <- c(0.98, 0.1);
+  }else{
+    K = state$K;
+    currenty = state$y;
+    V = state$V;
+    w = as.vector(state$w);
+    beta = state$beta;
+    sigma2 = state$sigma2;
+    alpha = state$alpha;
+    mu = as.vector(state$mu);
+    Sig = state$Sig;
+    theta = as.vector(state$theta);
+    spl0 = 500;
+    spS0 = spadapter*state$spSnew;
   }
-  theta = state$theta; if(is.null(theta)) theta <- c(0.95, 1);
-
+  
   #########################################################################################
   # calling the c++ code
   #########################################################################################
@@ -141,9 +160,9 @@ function (y,
                  nsave_ = nsave, 
                  nskip_ = nskip, 
                  ndisplay_ = ndisplay,
-                 y_ = y, 
+                 y_ = currenty, 
                  delta_ = delta, 
-                 X_ = as.matrix(X),
+                 X_ = as.matrix(t(X)),
                  N_ = N,
                  beta_ = beta, 
                  tau2_ = 1.0/sigma2,
@@ -171,6 +190,7 @@ function (y,
                  spadapter_ = spadapter,
                  dnm_ = dnm, dmm_=dmm, blocki_=blocki,
                  ds0m_= ds0m, ds0block_=ds0block,
+                 status_ = status,
                  PACKAGE = "spBayesSurv")
   } else{
     foo <- .Call("spCopulaDDP", 
@@ -178,9 +198,9 @@ function (y,
                  nsave_ = nsave, 
                  nskip_ = nskip, 
                  ndisplay_ = ndisplay,
-                 y_ = y, 
+                 y_ = currenty, 
                  delta_ = delta, 
-                 X_ = as.matrix(X),
+                 X_ = as.matrix(t(X)),
                  N_ = N,
                  beta_ = beta, 
                  tau2_ = 1.0/sigma2,
@@ -206,6 +226,7 @@ function (y,
                  spl0_ = spl0,
                  spS0_ = spS0, 
                  spadapter_ = spadapter,
+                 status_ = status,
                  PACKAGE = "spBayesSurv")
   }
   #########################################################################################
@@ -227,9 +248,10 @@ function (y,
                  ratetheta = foo$ratetheta,
                  cpo = foo$cpo,
                  Ypred = foo$Ypred,
-                 Zpred = foo$Zpred);
-  
-  cat("\n\n")
+                 Zpred = foo$Zpred,
+                 V = foo$V,
+                 K = foo$K,
+                 state=foo$state);
   class(output) <- c("spCopulaDDP")
   output
 }
\ No newline at end of file
