@@ -3,8 +3,11 @@
 
 #include <RcppArmadillo.h>
 
-#define ESMALL 1e-10  /* small number */
-#define ELARGE 1e+10 /* large number */
+#define ESMALL 1e-20  /* small number */
+#define ELARGE 1e+20 /* large number */
+#define SYSMIN 1e-305  /* small number */
+#define SYSMAX 1e+305 /* large number */
+#define LOGSYSMAX 702.28845336318397585 /* large number */
 typedef Rcpp::NumericMatrix::iterator mat_iterator;
 using namespace Rcpp;
 
